//---------------------------------------------------------------------------
// TestDetailedSubstrate
//---------------------------------------------------------------------------
#if !defined(TestDetailedSubstrate_H)
  #define TestDetailedSubstrate_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clGridBase;
class clSimManager;

/**
* This class tests substrate.
*/
class clTestDetailedSubstrate : public CppUnit::TestFixture
{

  public:

  /**
  * Groups the other substrate tests.
  */
  void TestDetailedSubstrate();

  protected:

  /**
  * Tests aspects of substrate grid setup.  This will enter two parameter
  * files:
  * <ol>
  * <li>With no grid maps defined
  * <li>With Substrate and substratecalcs grid maps
  * </ol>
  * In the first case, grid cell resolution should match harvest and the plot;
  * in the second, it should be what was specified in the map. The second case 
  * also tests that a value is correctly read in from the map.
  */
  void TestSubstrateGridSetup(clSimManager *p_oSimManager);
  
  
  /**
  * Tests new inputs to substrate.  A single tree is entered into each of 8 separate 
  * grid cells.  The trees are different combinations of adult/snag, standing/fallen, 
  * intact/newly broken/previously broken. We check to make sure the correct amount 
  * of new log and tipup is added to the cells after one timestep.  
  */
  void TestNewInputs(clSimManager *p_oSimManager);

  /**
  * Substrate normal processing run 1.  Verifies substrate decay calculations over 4 
  * timesteps with a 1-year timestep length.  This run has 5 cells with different harvest 
  * and mortality regimes. Non-directional tree fall and 100% decay class and tipup 
  * probabilities are used to avoid having to average over large numbers of trees. This 
  * function tests that the substrate numbers are calculated correctly for all types 
  * of logs, tipup, scarified soil, moss, and litter after each timestep. 
  */
  void TestSubstrateRun1(clSimManager *p_oSimManager);

  /**
  * Substrate normal processing, run 2.  This tests substrate calculations in Cell 1 
  * from run 1 (no harvest or mortality) with a 2-year timestep length this time.  Model is 
  * run for 1 timestep and final proportions are checked to match those after 2 timesteps
  * in run 1.  The numbers would not match for more complicated scenarios where mortality 
  * and harvest occur at different times, or where cohorts are removed over longer runs. 
  */
  void TestSubstrateRun2(clSimManager *p_oSimManager);

  /**
  * Verifies the substrate proportions for timestep 1 of TestSubstrateRun1().
  */
  void VerifyRun1TS1Proportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 2 of TestSubstrateRun1().
  */
  void VerifyRun1TS2Proportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 3 of TestSubstrateRun1().
  */
  void VerifyRun1TS3Proportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 4 of TestSubstrateRun1().
  */
  void VerifyRun1TS4Proportions(clGridBase *p_oSubstrateGrid);

  /**
  * Verifies the substrate proportions for timestep 1 of TestSubstrateRun2().
  */
  void VerifyRun2TS1Proportions(clGridBase *p_oSubstrateGrid);

 /**
  * Writes a parameter file for testing grid setup and normal processing runs 1 and 2.
  * @return Filename written.
  */
  char * WriteXMLFile1(int iYearsPerTS);

  /**
  * Writes a file for testing grid setup with map files.
  * @return Filename written.
  */
  char * WriteXMLFile2();


  /**
  * Writes a parameter file for testing new substrate inputs.
  * @return Filename written.
  */
  char * WriteXMLFile3();

};
//---------------------------------------------------------------------------
#endif // TestSubstrate
