//---------------------------------------------------------------------------
// TestMastingSpatialDisperse
//---------------------------------------------------------------------------
#if !defined(TestMastingSpatialDisperse_H)
  #define TestMastingSpatialDisperse_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clGridBase;
class clSimManager;

/**
  This class tests the masting spatial dispersal behavior 
  (clMastingSpatialDisperse).
*/
class clTestMastingSpatialDisperse : public CppUnit::TestFixture {

public:

  /**
  * Groups all tests into one.
  */
  void TestMastingSpatialDisperse();

private:

  /**
   * Tests first normal processing run. WriteXMLFile1() does setup. Three trees
   * are created for the plot. The file is run for one timestep and the 
   * seedlings are counted. A certain number of each species is expected. This
   * tests masting and mast grouping.
   * @param p_oSimManager Sim manager object.
   */
  void NormalProcessingRun1(clSimManager *p_oSimManager);
  
  /**
   * Tests second normal processing run. WriteXMLFile2() does setup. This is
   * the same as run 1 except it's three years per timestep.
   * @param p_oSimManager Sim manager object.
   */
  void NormalProcessingRun2(clSimManager *p_oSimManager);
  
  /**
   * Tests third normal processing run. WriteXMLFile3() does setup. This tests
   * fraction participating. It is just like run 1 except 100 trees are created
   * for each species and the final seedling counts should reflect the 
   * number of parents that participated.
   * @param p_oSimManager Sim manager object.
   */
  void NormalProcessingRun3(clSimManager *p_oSimManager);
  
  /**
   * Tests fourth normal processing run. WriteXMLFile4() does setup. This tests
   * STR draws per species vs. draws per tree. Four trees are used, 2 of each
   * of 2 species. Species 1 is a draw per species, Species 2 is draw per tree.
   * The two trees of species 1 should produce the same number of seedlings,
   * and the two trees of species 2 should produce different seedlings.
   * @param p_oSimManager Sim manager object.
   */
  void NormalProcessingRun4(clSimManager *p_oSimManager);
  
  /**
   * Tests fifth normal processing run. WriteXMLFile5() does setup. This tests
   * normal draws. 100 runs are done, and the mean and std deviation of the 
   * number of seedlings is calculated to see if it matches the requested.
   * @param p_oSimManager Sim manager object.
   */
  void NormalProcessingRun5(clSimManager *p_oSimManager);
  
  /**
   * Tests masting CDF setup. This uses WriteXMLCheckFile1() and 
   * WriteXMLCheckFile2() for setup. The first gives a CDF where the max time
   * until mast is before the end of the run. The second gives a CDF where the
   * max time until mast is the length of the run.
   */ 
  void CheckMastingSetup(clSimManager *p_oSimManager);

  /**
   * Makes sure errors are correctly caught on setup. All the error parameter 
   * files are fed in to make sure the errors are trapped.
   * @param p_oSimManager Sim manager object.
   */ 
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
   * Writes a parameter file for normal processing run 1.
   * @return Filename written.
   */
  char* WriteXMLFile1();
  
  /**
   * Writes a parameter file for normal processing run 2.
   * @return Filename written.
   */
  char* WriteXMLFile2();
  
  /**
   * Writes a parameter file for normal processing run 3.
   * @return Filename written.
   */
  char* WriteXMLFile3();
  
  /**
   * Writes a parameter file for normal processing run 4.
   * @return Filename written.
   */
  char* WriteXMLFile4();
  
  /**
   * Writes a parameter file for normal processing run 5.
   * @param iRandomSeed Random seed.
   * @return Filename written.
   */
  char* WriteXMLFile5(int iRandomSeed);
  
  /**
   * Writes a file to test masting CDF setup.
   * @return Filename written.
   */ 
  char* WriteXMLCheckFile1();
  
  /**
   * Writes a file to test masting CDF setup.
   * @return Filename written.
   */ 
  char* WriteXMLCheckFile2();
  
  /**
   * Writes a file where the masting "a" parameter is zero.
   * @return Filename written.
   */ 
  char* WriteXMLErrorFile1();
  
  /**
   * Writes a file where the masting fraction participating is not between
   * 0 and 1.
   * @return Filename written.
   */ 
  char* WriteXMLErrorFile2();
  
  /**
   * Writes a file where the non-masting fraction participating is not between
   * 0 and 1.
   * @return Filename written.
   */ 
  char* WriteXMLErrorFile3();
  
  /**
   * Writes a file where the masting beta is greater than 25.
   * @return Filename written.
   */ 
  char* WriteXMLErrorFile4();
  
  /**
   * Writes a file where the non-masting beta is greater than 25.
   * @return Filename written.
   */ 
  char* WriteXMLErrorFile5();
  
  /**
   * Writes a file where the masting theta is greater than 50.
   * @return Filename written.
   */ 
  char* WriteXMLErrorFile6();
  
  /**
   * Writes a file where the non-masting theta is greater than 50.
   * @return Filename written.
   */ 
  char* WriteXMLErrorFile7();
  
  /**
  * Writes tree stuff.
  * @param oOut File stream to write to.
  */
  void WriteTrees(std::fstream &oOut);
  
  /**
  * Writes allometry stuff.
  * @param oOut File stream to write to.
  */
  void WriteAllometry(std::fstream &oOut);
  
};
//---------------------------------------------------------------------------

#endif // TestMastingSpatialDisperse_H
