//---------------------------------------------------------------------------
// TestPlanting
//---------------------------------------------------------------------------
#if !defined(TestPlanting_H)
  #define TestPlanting_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>
#include <cppunit/TestSuite.h>

class clSimManager;
class clTreePopulation;

/**
* Tests the clPlant class.
*/
class clTestPlanting : public CppUnit::TestFixture
{
  public:

  /**
  * Groups together all the other plant testing functions into a comprehensive
  * plant test.
  */
  void TestPlanting();

  protected:

  /**
  * Do the first planting run.  WriteXMLFile1() is used to do the run.  The
  * characteristics of this run are:
  * <ul>
  * <li>Timestep 1
  * <li>Species 3, 4, 5
  * <li>Spacing Type - Random
  * <li>Density - 200/ha
  * <li>Grids - whole plot
  * <li>Amount - 33.3% for each species
  * <li>Seedling diam10 - default
  * </ul>
  * @param p_oSimManager Sim manager object.
  */
  void DoPlantingRun1(clSimManager *p_oSimManager);

  /**
  * Do the second planting run.  WriteXMLFile2() is used to do the run.  The
  * characteristics of this run are:
  * <ul>
  * <li>Timestep 2
  * <li>Species All
  * <li>Spacing Type - Gridded
  * <li>Density - 2 m
  * <li>Grids - whole plot
  * <li>Amount
  * <ul>
  * <li>Sp 0 - 5%
  * <li>Sp 1 - 10%
  * <li>Sp 2 - 15%
  * <li>Sp 3 - 30%
  * <li>Sp 4 - 1%
  * <li>Sp 5 - 10%
  * <li>Sp 6 - 7%
  * <li>Sp 7 - 8%
  * <li>Sp 8 - 4%
  * </ul>
  * <li>Seedling diam10
  * <ul>
  * <li>Sp 0 - 0.1
  * <li>Sp 1 - 0.2
  * <li>Sp 2 - 0.3
  * <li>Sp 3 - 0.4
  * <li>Sp 4 - 0.5
  * <li>Sp 5 - 0.6
  * <li>Sp 6 - 0.7
  * <li>Sp 7 - 0.8
  * <li>Sp 8 - 0.9
  * </ul>
  * </ul>
  * @param p_oSimManager Sim manager object.
  */
  void DoPlantingRun2(clSimManager *p_oSimManager);

  /**
  * Do the third planting run.  WriteXMLFile3() is used to do the run.  The
  * characteristics of this run are:
  * Planting event 1:
  * <ul>
  * <li>Timestep 1
  * <li>Species 0, 1, 2
  * <li>Spacing Type - Random
  * <li>Density - 150/ha
  * <li>Grids - 0,0-2-24; 5,0-5,24
  * <li>Amount
  * <ul>
  * <li>Sp 0 - 25%
  * <li>Sp 1 - 35%
  * <li>Sp 2 - 40%
  * </ul>
  * <li>Seedling diam10
  * <ul>
  * <li>Sp 0 - 0.1
  * <li>Sp 1 - 0.2
  * <li>Sp 2 - 0.3
  * </ul>
  * </ul>
  * Planting event 2:
  * <ul>
  * <li>Timestep 1
  * <li>Species 7, 8
  * <li>Spacing Type - Gridded
  * <li>Spacing - 3 m
  * <li>Grids - 0,0 - 24,2; 6,5-11,10
  * <li>Amount
  * <ul>
  * <li>Sp 7 - 40%
  * <li>Sp 8 - 60%
  * </ul>
  * <li>Seedling diam10
  * <ul>
  * <li>Sp 7 - 0.7
  * <li>Sp 8 - 0.8
  * </ul>
  * </ul>
  * @param p_oSimManager Sim manager object.
  */
  void DoPlantingRun3(clSimManager *p_oSimManager);

  /**
  * Do the fourth planting run.  WriteXMLFile4() is used to do the run.  The
  * characteristics of this run are:
  * Planting event 1:
  * <ul>
  * <li>Timestep 2
  * <li>Species - All
  * <li>Spacing Type - Random
  * <li>Density - 156.25/ha
  * <li>Grids - 24, 24
  * <li>Amount - 11.1% for each species
  * <li>Seedling diam10 - default
  * </ul>
  * Planting event 2:
  * <ul>
  * <li>Timestep 3
  * <li>Species - All
  * <li>Spacing Type - Gridded
  * <li>Spacing - 2 m
  * <li>Grids - 23, 23
  * <li>Amount - 11.1% for each species
  * <li>Seedling diam10 - default
  * </ul>
  * @param p_oSimManager Sim manager object.
  */
  void DoPlantingRun4(clSimManager *p_oSimManager);
  
  /**
   * Do the fifth planting run. This makes sure that random plantings happen 
   * correctly when grid cells do not divide evenly into the plot.
   * @param p_oSimManager Sim manager object.
   */
  void DoPlantingRun5(clSimManager *p_oSimManager);
  
  /**
   * Do the sixth planting run. This makes sure that gridded plantings happen 
   * correctly when grid cells do not divide evenly into the plot.
   * @param p_oSimManager Sim manager object.
   */
  void DoPlantingRun6(clSimManager *p_oSimManager);

  /**
  * Writes a parameter file to test planting for run 1.
  * @return Filename written.
  */
  char * WriteXMLFile1();

  /**
  * Writes a parameter file to test planting for run 2.
  * @return Filename written.
  */
  char * WriteXMLFile2();

  /**
  * Writes a parameter file to test planting for run 3.
  * @return Filename written.
  */
  char * WriteXMLFile3();
  
  /**
  * Writes a parameter file to test planting for run 4.
  * @return Filename written.
  */
  char * WriteXMLFile4();

  /**
  * Writes a parameter file to test planting for run 5.
  * @return Filename written.
  */
  char * WriteXMLFile5();
  
  /**
  * Writes a parameter file to test planting for run 6.
  * @return Filename written.
  */
  char * WriteXMLFile6();

};
//---------------------------------------------------------------------------
#endif // TestPlanting_H
