//---------------------------------------------------------------------------
// TestStormDamageApplier
//---------------------------------------------------------------------------
#if !defined(TestStormDamageApplier_H)
  #define TestStormDamageApplier_H

#include <cppunit/TestFixture.h>
#include <cppunit/TestCaller.h>

class clSimManager;
class clTreePopulation;

/**
* Tests the clStormDamageApplier class.
*/
class clTestStormDamageApplier : public CppUnit::TestFixture {

  public:

  /**
  * Assembles all other testing functions into one test.
  */
  void TestStormDamageApplier();

  /**
  * If you wish a visual test, this will output a spreadsheet for graph making.
  * It will write a tab-delimited file called "StormTest.txt".  There is one
  * table for each species.  It does storms at 0.02 severity intervals from 0
  * to 1, a thousand trees of each species, then outputs how many were
  * no damage, medium damage, and severe damage.
  */
  void MakeGraphs();

  protected:

  /**
  * Tests normal processing.  WriteXMLFile1() writes the parameter file.
  * Aspects of the run:
  * <ul>
  * <li>Grid resolution of "Storm Damage" is default.</li>
  * <li>Run is 5 timesteps.</li>
  * <li>1 year per timestep.</li>
  * <li>This behavior kills no trees.</li>
  * <li>There are no snags for this behavior.</li>
  * <li>The following trees will be added, with the appropriate storm triggered
  * for timestep 1:
  * <ul>
  * <li>Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.  Damage index is
  * 0.</li>
  * <li>Grid (0,1): 1000 trees of Species 1 and 2, DBH 30.  Damage index is
  * 0.5.</li>
  * <li>Grid (0,2): 1000 trees of Species 1 and 2, DBH 30.  Damage index is
  * 1.</li>
  * <li>Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.  Damage index is
  * 0.</li>
  * <li>Grid (1,1): 1000 trees of Species 1 and 2, DBH 8.  Damage index is
  * 0.3.</li>
  * <li>Grid (1,2): 1000 trees of Species 1 and 2, DBH 8.  Damage index is
  * 1.</li>
  * <li>Grid (2,0): 1000 trees of Species 1 and 2, DBH 45.  Damage index is 0.2.
  * Initial counter is 1002.</li>
  * <li>Grid (2,1): 1000 trees of Species 1 and 2, DBH 15.  Damage index is 0.7.
  * Initial counter is 2003.</li>
  * <li>All grids get 100 trees of DBH 4, Species 1, and DBH 5, Species 2.</li>
  * </ul></li>
  * </ul>
  *
  * The run proceeds for 5 timesteps and the number of trees with each damage
  * counter is checked through time.
  *
  * @param p_oSimManager Sim Manager object to run our test run.
  */
  void TestNormalProcessingRun1(clSimManager *p_oSimManager);

  /**
  * Tests normal processing.  WriteXMLFile2() writes the parameter file.
  * Aspects of the run:
  * <ul>
  * <li>Grid resolution of "Storm Damage" is default.</li>
  * <li>Run is 3 timesteps.</li>
  * <li>3 years per timestep.</li>
  * <li>This behavior kills no trees.</li>
  * <li>There are no snags for this behavior.</li>
  * <li>The following trees will be added, with the appropriate storm triggered
  * for timestep 1:
  * <ul>
  * <li>Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.  Damage index is
  * 0.</li>
  * <li>Grid (0,1): 1000 trees of Species 1 and 2, DBH 30.  Damage index is
  * 0.5.</li>
  * <li>Grid (0,2): 1000 trees of Species 1 and 2, DBH 30.  Damage index is
  * 1.</li>
  * <li>Grid (1,0): 1000 trees of Species 1 and 2, DBH 8.  Damage index is
  * 0.</li>
  * <li>Grid (1,1): 1000 trees of Species 1 and 2, DBH 8.  Damage index is
  * 0.3.</li>
  * <li>Grid (1,2): 1000 trees of Species 1 and 2, DBH 8.  Damage index is
  * 1.</li>
  * <li>Grid (2,0): 1000 trees of Species 1 and 2, DBH 45.  Damage index is 0.2.
  * Initial counter is 1002.</li>
  * <li>Grid (2,1): 1000 trees of Species 1 and 2, DBH 15.  Damage index is 0.7.
  * Initial counter is 2003.</li>
  * <li>All grids get 100 trees of DBH 4, Species 1, and DBH 5, Species 2.</li>
  * </ul></li>
  * </ul>
  *
  * The run proceeds for 3 timesteps and the number of trees with each damage
  * counter is checked through time.
  *
  * @param p_oSimManager Sim Manager object to run our test run.
  */
  void TestNormalProcessingRun2(clSimManager *p_oSimManager);
  
  /**
  * Tests normal processing.  WriteXMLFile1() writes the parameter file. Tests
  * multiple storms in one timestep.
  * Aspects of the run:
  * <ul>
  * <li>Grid resolution of "Storm Damage" is default.</li>
  * <li>Run is 5 timesteps.</li>
  * <li>1 year per timestep.</li>
  * <li>This behavior kills no trees.</li>
  * <li>There are no snags for this behavior.</li>
  * <li>The following trees will be added, with the appropriate storm triggered
  * for timestep 1:
  * <ul>
  * <li>Grid (0,0): 1000 trees of Species 1 and 2, DBH 30.  Damage index is 0 
  * and 0.</li>
  * <li>Grid (0,1): 1000 trees of Species 1 and 2, DBH 30.  Damage index is 0.5
  * and 0.7.</li>
  * <li>Grid (0,2): 1000 trees of Species 1 and 2, DBH 30.  Damage index is 1 
  * and 0.</li>
  * <li>Grid (1,0): 1000 trees of Species 1 and 2, DBH 30.  Damage index is 0 
  * and 0.2.</li>
  * <li>Grid (1,1): 1000 trees of Species 1 and 2, DBH 30.  Damage index is 0.5
  * and 0.3.</li>
  * <li>Grid (1,2): 1000 trees of Species 1 and 2, DBH 30.  Damage index is 1
  * and 0.8.</li>
  * <li>All grids get 100 trees of DBH 4, Species 1, and DBH 5, Species 2.</li>
  * </ul></li>
  * </ul>
  *
  * The run proceeds for 5 timesteps and the number of trees with each damage
  * counter is checked through time.
  *
  * @param p_oSimManager Sim Manager object to run our test run.
  */
  void TestNormalProcessingRun3(clSimManager *p_oSimManager);

  /**
  * Checks the expected results for timestep 1 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep1(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 2 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep2(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 3 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep3(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 4 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep4(clTreePopulation *p_oPop);

  /**
  * Checks the expected results for timestep 5 of TestNormalProcessingRun1().
  * @param p_oPop Tree population object.
  */
  void TestRun1Timestep5(clTreePopulation *p_oPop);

  /**
  * Tests error processing.
  */
  void TestErrorProcessing(clSimManager *p_oSimManager);

  /**
  * Writes the parameter file for TestNormalProcessing1().
  * @return File name of the freshly written parameter file.
  */
  char *WriteXMLFile1();

  /**
  * Writes the parameter file for TestNormalProcessing2().
  * @return File name of the freshly written parameter file.
  */
  char *WriteXMLFile2();

  /**
  * Writes the parameter file for MakeGraphs().
  * @return File name of the freshly written parameter file.
  */
  char *WriteXMLFile3();

  /**
  * Writes an error file where the storm behavior is not also enabled.
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile1();

  /**
  * Writes an error file where the storm damage applier behavior is applied
  * to seedlings.
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile2();

  /**
  * Writes an error file where the value for number of years to heal is
  * not positive.
  * @return File name of the freshly written parameter file.
  */
  char* WriteXMLErrorFile3();

  /**
  * Writes common portions of the test parameter files.  Starts with
  * &lt;randomSeed&gt; and ends with &lt;/allometry&gt;.
  * @param oOut File stream to write to.
  */
  void WriteCommonStuff(std::fstream &oOut);

  /**
  * Writes the behavior list for the test parameter files.
  * @param oOut File stream to write to.
  */
  void WriteBehaviorList(std::fstream &oOut);

};
//---------------------------------------------------------------------------
#endif //TestStormDamageApplier_H
